/* Mastermind-Bibliothek.
 *
 * Stefan Arnold XI.03
 */


#pragma once

/* Anzahl der Zeichen in einem Code */
#ifndef CODELAENGE
    #define CODELAENGE 4
#endif

/* maximale Anzahl an Rateschritten */
#ifndef MAX_SCHRITTE
    #define MAX_SCHRITTE 10
#endif

/* groesste in einem Code erlaubte Zahl */
#ifndef MAX_OPTIONEN
    #define MAX_OPTIONEN 10
#endif

/* Zahlen, aus denen Codes gebildet werden */
typedef unsigned int zahl;

/* Geheimcodes */
typedef zahl code[CODELAENGE];

/* Bewertung eines Codes */
typedef struct{
    zahl schwarz,weiss;
}bewertung;

/* Indices fuer Mengenelemente */
typedef unsigned int index;

/* (geordnete) Menge und Anzahl an Elementplaetzen. */
typedef struct{
    unsigned char* elem;
    index size;
}menge;

/* Wertigkeiten der einzelnen Stellen im Code bei
   der Umrechnung in Dezimalzahlen */
typedef unsigned int wertigkeit[CODELAENGE];


/* Liefert eine Bewertung des angegebenen Tips bei gegebenem Code
 * nach den auf Uebungsblatt 5 spezifizierten Regeln.
 */
bewertung getBewertung(code Code,code Tip,zahl Optionen);

/* Gibt die im angegebenen Code enthaltenen Zahlen aus.
 */
void printCode(code Code);

/* Gibt die angegebene Bewertung aus.
 */
void printBewertung(bewertung b);

/* Liest Daten vom Typ 'code' ein. Die Zahlen muessen dabei
 * im Bereich 0..'maxZahl' liegen. Liefert...
 *   0, falls das Einlesen erfolgreich war,
 *  das Ergebnis von 'scanf',
 *      wenn ein unbekannter Fehler
 *      aufgetreten ist,
 *  1, falls eine Zahl groesser als 'maxZahl' war
 *      oder nicht genuegend Zahlen eingelesen wurden,
 *      oder eine Zahl doppelt eingegeben wurde, obwohl
 *      ohne Vielfachheiten gespielt wird.
 *   .
 */
int readCode(code Code,zahl Optionen,int mitVielfachheiten);

/* Liest Daten vom Typ 'bewertung' ein. Die Summe von
 * 'schwarz' und 'weiss' muss dabei im Bereich 0..CODELAENGE
 * liegen. Weitere Kontrollen finden nicht statt. Liefert...
 *   0, falls das Einlesen erfolgreich war,
 *  das Ergebnis von 'scanf',
 *      wenn ein unbekannter Fehler
 *      aufgetreten ist,
 *  1, falls die Summe von 'schwarz' und 'weiss' ausserhalb
 *      des gueltigen Bereichs liegt,
 *      oder nicht genuegend Zahlen eingelesen wurden.
 *   .
 */
int readBewertung(bewertung* b);

/* Berechnet die Wertigkeiten, die zum Extrahieren von Codes benoetigt
 * werden.
 */
void berechneWertigkeiten(wertigkeit W,zahl Optionen);

/* Berechnet den zum angegebenen Index gehoerenden Code.
 */
void extrahiereCode(index src,code dst,wertigkeit W,zahl Optionen);

/* Erzeugt einen zufaelligen Code.
 */
void ZufallsCode(code Code,zahl Optionen,int mitVielfachheiten);

/* Erwartungswert der verbleibenden noetigen Tips, wenn der
 * angegebene Tip gewaehlt wird.
 */
int Erwartungswert(code Tip,menge M,wertigkeit W,zahl Optionen);

/* Berechnet einen der Tips, fuer die 'Erwartungswert' ein
 * (nichtnegatives) Minimum annimmt und schreibt diesen in
 * 'dst'.
 * Liefert den zugehoerigen Erwartungswert verbleibender Tips
 * oder -1, falls kein Tip mehr moeglich ist.
 */
int berechneTip(code Tip,menge M,zahl Optionen);

/* Entfernt alle Elemente aus M, die aufgrund des angegebenen
 * Tips und der Bewertung keine moeglichen Loesungen mehr sind.
 */
void berechneVerbleibende(menge M,code Tip,bewertung b,zahl Optionen);

/* Liefert genau dann 0, wenn i kein Element von M ist.
 */
__inline int istElement(index i,menge M);

/* Ist i Element von M, wird es entfernt, sonst hinzugefuegt.
 */
__inline void invertiereElement(index i,menge M);
